"""
This module contains the asynchronous API to Obelisk-py.
These methods all return a :any:`Coroutine`.

Relevant entrance points are :class:`.producer.Producer` to publish data to Obelisk,
or :class:`.consumer.Consumer` to consume data.
"""
